@@ -1,8 +1,10 @@
 # coding: utf8
+
 from __future__ import unicode_literals
 import itertools
-import string
 import os
+import pydoc
+import string
 import sys
 from contextlib import contextmanager
 
@@ -14,15 +16,14 @@ from bpython.test import mock
 from bpython import config, inspection
 from bpython.curtsiesfrontend.repl import BaseRepl
 from bpython.curtsiesfrontend import replpainter
-from bpython.repl import History
 from bpython.curtsiesfrontend.repl import INCONSISTENT_HISTORY_MSG, \
     CONTIGUITY_BROKEN_MSG
-from bpython.test import FixLanguageTestCase as TestCase
+from bpython.test import FixLanguageTestCase as TestCase, TEST_CONFIG
 
 
 def setup_config():
     config_struct = config.Struct()
-    config.loadini(config_struct, os.devnull)
+    config.loadini(config_struct, TEST_CONFIG)
     config_struct.cli_suggestion_width = 1
     return config_struct
 
@@ -47,8 +48,6 @@ class CurtsiesPaintingTest(FormatStringTest, ClearEnviron):
             def _request_refresh(inner_self):
                 pass
         self.repl = TestRepl(config=setup_config())
-        # clear history
-        self.repl.rl_history = History()
         self.repl.height, self.repl.width = (5, 10)
 
     @property
@@ -136,6 +135,43 @@ class TestCurtsiesPaintingSimple(CurtsiesPaintingTest):
                             'Also has side effects'])
         self.assertFSArraysEqualIgnoringFormatting(actual, expected)
 
+    def test_unicode_docstrings(self):
+        "A bit of a special case in Python 2"
+        # issue 653
+
+        def foo():
+            u"åß∂ƒ"
+
+        actual = replpainter.formatted_docstring(
+                     foo.__doc__, 40, config=setup_config())
+        expected = fsarray([u'åß∂ƒ'])
+        self.assertFSArraysEqualIgnoringFormatting(actual, expected)
+
+    def test_nonsense_docstrings(self):
+        for docstring in [123, {}, [], ]:
+            try:
+                replpainter.formatted_docstring(
+                    docstring, 40, config=setup_config())
+            except Exception:
+                self.fail('bad docstring caused crash: {!r}'.format(docstring))
+
+    def test_weird_boto_docstrings(self):
+        # Boto does something like this.
+        # botocore: botocore/docs/docstring.py
+        class WeirdDocstring(str):
+            # a mighty hack. See botocore/docs/docstring.py
+            def expandtabs(self, tabsize=8):
+                return u'asdfåß∂ƒ'.expandtabs(tabsize)
+
+        def foo():
+            pass
+
+        foo.__doc__ = WeirdDocstring()
+        wd = pydoc.getdoc(foo)
+        actual = replpainter.formatted_docstring(wd, 40, config=setup_config())
+        expected = fsarray([u'asdfåß∂ƒ'])
+        self.assertFSArraysEqualIgnoringFormatting(actual, expected)
+
     def test_paint_lasts_events(self):
         actual = replpainter.paint_last_events(4, 100, ['a', 'b', 'c'],
                                                config=setup_config())
@@ -198,8 +234,6 @@ class HigherLevelCurtsiesPaintingTest(CurtsiesPaintingTest):
             def _request_refresh(inner_self):
                 self.refresh()
         self.repl = TestRepl(banner='', config=setup_config())
-        # clear history
-        self.repl.rl_history = History()
         self.repl.height, self.repl.width = (5, 32)
 
     def send_key(self, key):
